from io import BytesIO
from zipfile import ZIP_DEFLATED, ZipFile

from .Document import Document
from .ImportConfiguration import ImportConfiguration


class Submission:
    __document: Document
    __importConfiguration: ImportConfiguration

    def __init__(
        self,
        o: dict = None,
        *,
        document: Document = None,
        importConfiguration: ImportConfiguration = None,
    ):
        if isinstance(o, Document):
            # Assign attributes from object without typechecking
            self.__document = o.document
            self.__importConfiguration = o.importConfiguration
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.document = o.get("document")
            self.importConfiguration = o.get("importConfiguration")
        else:
            # Assign attributes from named keywords with typechecking
            self.document = document
            self.importConfiguration = importConfiguration

    @property
    def document(self) -> Document:
        return self.__document

    @document.setter
    def document(self, val: Document) -> None:
        if val is not None and not isinstance(val, Document):
            raise TypeError("Attribute 'document' must be a Document object.")
        self.__document = None if val is None else Document(val)

    @property
    def importConfiguration(self) -> ImportConfiguration:
        return self.__importConfiguration

    @importConfiguration.setter
    def importConfiguration(self, val: ImportConfiguration) -> None:
        if val is not None and not isinstance(val, ImportConfiguration):
            raise TypeError(
                "Attribute 'importConfiguration' must be a ImportConfiguration object."
            )
        self.__importConfiguration = None if val is None else ImportConfiguration(val)

    #  def importData(self, data:List, *, importConfiguration:ImportConfiguration=None):
    #    ic = self.__importConfiguration if importConfiguration is None else importConfiguration
    #    for row in data:
    #      pass

    #  def importResult(self, result: dict):
    #      pass

    def generateZIP(self, fileName: str = None):
        if not isinstance(fileName, str):
            raise TypeError("Parameter 'fileName' must be a string.")

        mem = BytesIO()
        zip = ZipFile(mem, mode="w", compression=ZIP_DEFLATED)

        results = self.__document.generateXML()
        zip.writestr("results.xml", results)

        # TODO: Add attachment files, if necessary. Example:
        #   zip.writestr('rawdata.csv', self.data)

        zip.close()
        mem.seek(0)

        with open(fileName, "wb") as out:
            out.write(mem.read())



from typing import List, Union

from yattag import Doc

from .DetectionQuantitationLimit import DetectionQuantitationLimit
from .SimpleContent import (
    AnalysisEndDate,
    AnalysisStartDate,
    LaboratoryAccreditationAuthorityName,
    LaboratoryAccreditationIndicator,
    LaboratoryCommentText,
    LaboratoryName,
    LaboratorySampleSplitRatio,
    TaxonomistAccreditationAuthorityName,
    TaxonomistAccreditationIndicator,
)
from .WQXTime import WQXTime


class ResultLabInformation:
    """
    Describes information obtained by a laboratory related to a specific laboratory analysis.
    """

    __laboratoryName: LaboratoryName
    __analysisStartDate: AnalysisStartDate
    __analysisStartTime: WQXTime
    __analysisEndDate: AnalysisEndDate
    __analysisEndTime: WQXTime
    __laboratoryCommentText: LaboratoryCommentText
    __resultDetectionQuantitationLimit: List[DetectionQuantitationLimit]
    __laboratorySampleSplitRatio: LaboratorySampleSplitRatio
    __laboratoryAccreditationIndicator: LaboratoryAccreditationIndicator
    __laboratoryAccreditationAuthorityName: LaboratoryAccreditationAuthorityName
    __taxonomistAccreditationIndicator: TaxonomistAccreditationIndicator
    __taxonomistAccreditationAuthorityName: TaxonomistAccreditationAuthorityName

    def __init__(
        self,
        o: dict = None,
        *,
        laboratoryName: LaboratoryName = None,
        analysisStartDate: AnalysisStartDate = None,
        analysisStartTime: WQXTime = None,
        analysisEndDate: AnalysisEndDate = None,
        analysisEndTime: WQXTime = None,
        laboratoryCommentText: LaboratoryCommentText = None,
        resultDetectionQuantitationLimit: DetectionQuantitationLimit = None,
        laboratorySampleSplitRatio: LaboratorySampleSplitRatio = None,
        laboratoryAccreditationIndicator: LaboratoryAccreditationIndicator = None,
        laboratoryAccreditationAuthorityName: LaboratoryAccreditationAuthorityName = None,
        taxonomistAccreditationIndicator: TaxonomistAccreditationIndicator = None,
        taxonomistAccreditationAuthorityName: TaxonomistAccreditationAuthorityName = None
    ):
        if isinstance(o, ResultLabInformation):
            # Assign attributes from object without typechecking
            self.__laboratoryName = o.laboratoryName
            self.__analysisStartDate = o.analysisStartDate
            self.__analysisStartTime = o.analysisStartTime
            self.__analysisEndDate = o.analysisEndDate
            self.__analysisEndTime = o.analysisEndTime
            self.__laboratoryCommentText = o.laboratoryCommentText
            self.__resultDetectionQuantitationLimit = o.resultDetectionQuantitationLimit
            self.__laboratorySampleSplitRatio = o.laboratorySampleSplitRatio
            self.__laboratoryAccreditationIndicator = o.laboratoryAccreditationIndicator
            self.__laboratoryAccreditationAuthorityName = (
                o.laboratoryAccreditationAuthorityName
            )
            self.__taxonomistAccreditationIndicator = o.taxonomistAccreditationIndicator
            self.__taxonomistAccreditationAuthorityName = (
                o.taxonomistAccreditationAuthorityName
            )
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.laboratoryName = o.get("laboratoryName")
            self.analysisStartDate = o.get("analysisStartDate")
            self.analysisStartTime = o.get("analysisStartTime")
            self.analysisEndDate = o.get("analysisEndDate")
            self.analysisEndTime = o.get("analysisEndTime")
            self.laboratoryCommentText = o.get("laboratoryCommentText")
            self.resultDetectionQuantitationLimit = o.get(
                "resultDetectionQuantitationLimit"
            )
            self.laboratorySampleSplitRatio = o.get("laboratorySampleSplitRatio")
            self.laboratoryAccreditationIndicator = o.get(
                "laboratoryAccreditationIndicator"
            )
            self.laboratoryAccreditationAuthorityName = o.get(
                "laboratoryAccreditationAuthorityName"
            )
            self.taxonomistAccreditationIndicator = o.get(
                "taxonomistAccreditationIndicator"
            )
            self.taxonomistAccreditationAuthorityName = o.get(
                "taxonomistAccreditationAuthorityName"
            )
        else:
            # Assign attributes from named keywords with typechecking
            self.laboratoryName = laboratoryName
            self.analysisStartDate = analysisStartDate
            self.analysisStartTime = analysisStartTime
            self.analysisEndDate = analysisEndDate
            self.analysisEndTime = analysisEndTime
            self.laboratoryCommentText = laboratoryCommentText
            self.resultDetectionQuantitationLimit = resultDetectionQuantitationLimit
            self.laboratorySampleSplitRatio = laboratorySampleSplitRatio
            self.laboratoryAccreditationIndicator = laboratoryAccreditationIndicator
            self.laboratoryAccreditationAuthorityName = (
                laboratoryAccreditationAuthorityName
            )
            self.taxonomistAccreditationIndicator = taxonomistAccreditationIndicator
            self.taxonomistAccreditationAuthorityName = (
                taxonomistAccreditationAuthorityName
            )

    @property
    def laboratoryName(self) -> LaboratoryName:
        return self.__laboratoryName

    @laboratoryName.setter
    def laboratoryName(self, val: LaboratoryName) -> None:
        self.__laboratoryName = None if val is None else LaboratoryName(val)

    @property
    def analysisStartDate(self) -> AnalysisStartDate:
        return self.__analysisStartDate

    @analysisStartDate.setter
    def analysisStartDate(self, val: AnalysisStartDate) -> None:
        self.__analysisStartDate = None if val is None else AnalysisStartDate(val)

    @property
    def analysisStartTime(self) -> WQXTime:
        """
        The local time and relative time zone when the analysis began.
        """
        return self.__analysisStartTime

    @analysisStartTime.setter
    def analysisStartTime(self, val: WQXTime) -> None:
        """
        The local time and relative time zone when the analysis began.
        """
        self.__analysisStartTime = None if val is None else WQXTime(val)

    @property
    def analysisEndDate(self) -> AnalysisEndDate:
        return self.__analysisEndDate

    @analysisEndDate.setter
    def analysisEndDate(self, val: AnalysisEndDate) -> None:
        self.__analysisEndDate = None if val is None else AnalysisEndDate(val)

    @property
    def analysisEndTime(self) -> WQXTime:
        """
        The local time and relative time zone when the analysis was finished.
        """
        return self.__analysisEndTime

    @analysisEndTime.setter
    def analysisEndTime(self, val: WQXTime) -> None:
        """
        The local time and relative time zone when the analysis was finished.
        """
        self.__analysisEndTime = None if val is None else WQXTime(val)

    @property
    def laboratoryCommentText(self) -> LaboratoryCommentText:
        return self.__laboratoryCommentText

    @laboratoryCommentText.setter
    def laboratoryCommentText(self, val: LaboratoryCommentText) -> None:
        self.__laboratoryCommentText = None if val is None else LaboratoryCommentText(val)

    @property
    def resultDetectionQuantitationLimit(self) -> List[DetectionQuantitationLimit]:
        """
        Information that describes one of a variety of detection or quantitation limits
        determined in a laboratory.
        """
        return self.__resultDetectionQuantitationLimit

    @resultDetectionQuantitationLimit.setter
    def resultDetectionQuantitationLimit(
        self, val: Union[DetectionQuantitationLimit, List[DetectionQuantitationLimit]]
    ) -> None:
        """
        Information that describes one of a variety of detection or quantitation limits
        determined in a laboratory.
        """
        if val is None:
            self.__resultDetectionQuantitationLimit = []
        elif isinstance(val, list):
            r: List[DetectionQuantitationLimit] = []
            for x in val:
                r.append(DetectionQuantitationLimit(x))
            self.__resultDetectionQuantitationLimit = r
        else:
            self.__resultDetectionQuantitationLimit = [DetectionQuantitationLimit(val)]

    @property
    def laboratorySampleSplitRatio(self) -> LaboratorySampleSplitRatio:
        return self.__laboratorySampleSplitRatio

    @laboratorySampleSplitRatio.setter
    def laboratorySampleSplitRatio(self, val: LaboratorySampleSplitRatio) -> None:
        self.__laboratorySampleSplitRatio = (
            None if val is None else LaboratorySampleSplitRatio(val)
        )

    @property
    def laboratoryAccreditationIndicator(self) -> LaboratoryAccreditationIndicator:
        return self.__laboratoryAccreditationIndicator

    @laboratoryAccreditationIndicator.setter
    def laboratoryAccreditationIndicator(
        self, val: LaboratoryAccreditationIndicator
    ) -> None:
        self.__laboratoryAccreditationIndicator = (
            None if val is None else LaboratoryAccreditationIndicator(val)
        )

    @property
    def laboratoryAccreditationAuthorityName(
        self,
    ) -> LaboratoryAccreditationAuthorityName:
        return self.__laboratoryAccreditationAuthorityName

    @laboratoryAccreditationAuthorityName.setter
    def laboratoryAccreditationAuthorityName(
        self, val: LaboratoryAccreditationAuthorityName
    ) -> None:
        self.__laboratoryAccreditationAuthorityName = (
            None if val is None else LaboratoryAccreditationAuthorityName(val)
        )

    @property
    def taxonomistAccreditationIndicator(self) -> TaxonomistAccreditationIndicator:
        return self.__taxonomistAccreditationIndicator

    @taxonomistAccreditationIndicator.setter
    def taxonomistAccreditationIndicator(
        self, val: TaxonomistAccreditationIndicator
    ) -> None:
        self.__taxonomistAccreditationIndicator = (
            None if val is None else TaxonomistAccreditationIndicator(val)
        )

    @property
    def taxonomistAccreditationAuthorityName(
        self,
    ) -> TaxonomistAccreditationAuthorityName:
        return self.__taxonomistAccreditationAuthorityName

    @taxonomistAccreditationAuthorityName.setter
    def taxonomistAccreditationAuthorityName(
        self, val: TaxonomistAccreditationAuthorityName
    ) -> None:
        self.__taxonomistAccreditationAuthorityName = (
            None if val is None else TaxonomistAccreditationAuthorityName(val)
        )

    def generateXML(self, name: str = "ResultLabInformation") -> str:  # noqa: C901
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__laboratoryName is not None:
                line("LaboratoryName", self.__laboratoryName)
            if self.__analysisStartDate is not None:
                line("AnalysisStartDate", self.__analysisStartDate)
            if self.__analysisStartTime is not None:
                asis(self.__analysisStartTime.generateXML("AnalysisStartTime"))
            if self.__analysisEndDate is not None:
                line("AnalysisEndDate", self.__analysisEndDate)
            if self.__analysisEndTime is not None:
                asis(self.__analysisEndTime.generateXML("AnalysisEndTime"))
            if self.__laboratoryCommentText is not None:
                line("LaboratoryCommentText", self.__laboratoryCommentText)
            for x in self.__resultDetectionQuantitationLimit:
                asis(x.generateXML("ResultDetectionQuantitationLimit"))
            if self.__laboratorySampleSplitRatio is not None:
                line("LaboratorySampleSplitRatio", self.__laboratorySampleSplitRatio)
            if self.__laboratoryAccreditationIndicator is not None:
                line(
                    "LaboratoryAccreditationIndicator",
                    self.__laboratoryAccreditationIndicator,
                )
            if self.__laboratoryAccreditationAuthorityName is not None:
                line(
                    "LaboratoryAccreditationAuthorityName",
                    self.__laboratoryAccreditationAuthorityName,
                )
            if self.__taxonomistAccreditationIndicator is not None:
                line(
                    "TaxonomistAccreditationIndicator",
                    self.__taxonomistAccreditationIndicator,
                )
            if self.__taxonomistAccreditationAuthorityName is not None:
                line(
                    "TaxonomistAccreditationAuthorityName",
                    self.__taxonomistAccreditationAuthorityName,
                )

        return doc.getvalue()



from typing import List, Union

from yattag import Doc

from ..exceptions import WQXException
from .Activity import Activity
from .ActivityGroup import ActivityGroup
from .BiologicalHabitatIndex import BiologicalHabitatIndex
from .ElectronicAddress import ElectronicAddress
from .MonitoringLocation import MonitoringLocation
from .OrganizationAddress import OrganizationAddress
from .OrganizationDescription import OrganizationDescription
from .Project import Project
from .Telephonic import Telephonic


class Organization:
    """
    Schema used to transfer organization information.
    """

    __organizationDescription: OrganizationDescription
    __electronicAddress: List[ElectronicAddress]
    __telephonic: List[Telephonic]
    __organizationAddress: List[OrganizationAddress]
    __project: List[Project]
    __monitoringLocation: List[MonitoringLocation]
    __biologicalHabitatIndex: List[BiologicalHabitatIndex]
    __activity: List[Activity]
    __activityGroup: List[ActivityGroup]

    def __init__(
        self,
        o: dict = None,
        *,
        organizationDescription: OrganizationDescription = None,
        electronicAddress: List[ElectronicAddress] = None,
        telephonic: List[Telephonic] = None,
        organizationAddress: List[OrganizationAddress] = None,
        project: List[Project] = None,
        monitoringLocation: List[MonitoringLocation] = None,
        biologicalHabitatIndex: List[BiologicalHabitatIndex] = None,
        activity: List[Activity] = None,
        activityGroup: List[ActivityGroup] = None
    ):
        if isinstance(o, Organization):
            # Assign attributes from object without typechecking
            self.__organizationDescription = o.organizationDescription
            self.__electronicAddress = o.electronicAddress
            self.__telephonic = o.telephonic
            self.__organizationAddress = o.organizationAddress
            self.__project = o.project
            self.__monitoringLocation = o.monitoringLocation
            self.__biologicalHabitatIndex = o.biologicalHabitatIndex
            self.__activity = o.activity
            self.__activityGroup = o.activityGroup
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.organizationDescription = o.get("organizationDescription")
            self.electronicAddress = o.get("electronicAddress")
            self.telephonic = o.get("telephonic")
            self.organizationAddress = o.get("organizationAddress")
            self.project = o.get("project")
            self.monitoringLocation = o.get("monitoringLocation")
            self.biologicalHabitatIndex = o.get("biologicalHabitatIndex")
            self.activity = o.get("activity")
            self.activityGroup = o.get("activityGroup")
        else:
            # Assign attributes from named keywords with typechecking
            self.organizationDescription = organizationDescription
            self.electronicAddress = electronicAddress
            self.telephonic = telephonic
            self.organizationAddress = organizationAddress
            self.project = project
            self.monitoringLocation = monitoringLocation
            self.biologicalHabitatIndex = biologicalHabitatIndex
            self.activity = activity
            self.activityGroup = activityGroup

    @property
    def organizationDescription(self) -> OrganizationDescription:
        return self.__organizationDescription

    @organizationDescription.setter
    def organizationDescription(self, val: OrganizationDescription) -> None:
        if val is not None and not isinstance(val, OrganizationDescription):
            raise WQXException(
                "Property 'organizationDescription' must be an OrganizationDescription "
                "object."
            )
        self.__organizationDescription = (
            None if val is None else OrganizationDescription(val)
        )

    @property
    def electronicAddress(self) -> List[ElectronicAddress]:
        return self.__electronicAddress

    @electronicAddress.setter
    def electronicAddress(
        self, val: Union[ElectronicAddress, List[ElectronicAddress]]
    ) -> None:
        if val is None:
            self.__electronicAddress = []
        elif isinstance(val, list):
            r: List[ElectronicAddress] = []
            for x in val:
                r.append(ElectronicAddress(x))
            self.__electronicAddress = r
        else:
            self.__electronicAddress = [ElectronicAddress(val)]

    @property
    def telephonic(self) -> List[Telephonic]:
        return self.__telephonic

    @telephonic.setter
    def telephonic(self, val: Union[Telephonic, List[Telephonic]]) -> None:
        if val is None:
            self.__telephonic = []
        elif isinstance(val, list):
            r: List[Telephonic] = []
            for x in val:
                r.append(Telephonic(x))
            self.__telephonic = r
        else:
            self.__telephonic = [Telephonic(val)]

    @property
    def organizationAddress(self) -> List[OrganizationAddress]:
        return self.__organizationAddress

    @organizationAddress.setter
    def organizationAddress(
        self, val: Union[OrganizationAddress, List[OrganizationAddress]]
    ) -> None:
        if val is None:
            self.__organizationAddress = []
        elif isinstance(val, list):
            r: List[OrganizationAddress] = []
            for x in val:
                r.append(OrganizationAddress(x))
            self.__organizationAddress = r
        else:
            self.__organizationAddress = [OrganizationAddress(val)]

    @property
    def project(self) -> List[Project]:
        return self.__project

    @project.setter
    def project(self, val: Union[Project, List[Project]]) -> None:
        if val is None:
            self.__project = []
        elif isinstance(val, list):
            r: List[Project] = []
            for x in val:
                r.append(Project(x))
            self.__project = r
        else:
            self.__project = [Project(val)]

    @property
    def monitoringLocation(self) -> List[MonitoringLocation]:
        return self.__monitoringLocation

    @monitoringLocation.setter
    def monitoringLocation(
        self, val: Union[MonitoringLocation, List[MonitoringLocation]]
    ) -> None:
        if val is None:
            self.__monitoringLocation = []
        elif isinstance(val, list):
            r: List[MonitoringLocation] = []
            for x in val:
                r.append(MonitoringLocation(x))
            self.__monitoringLocation = r
        else:
            self.__monitoringLocation = [MonitoringLocation(val)]

    @property
    def biologicalHabitatIndex(self) -> List[BiologicalHabitatIndex]:
        return self.__biologicalHabitatIndex

    @biologicalHabitatIndex.setter
    def biologicalHabitatIndex(
        self, val: Union[BiologicalHabitatIndex, List[BiologicalHabitatIndex]]
    ) -> None:
        if val is None:
            self.__biologicalHabitatIndex = []
        elif isinstance(val, list):
            r: List[BiologicalHabitatIndex] = []
            for x in val:
                r.append(BiologicalHabitatIndex(x))
            self.__biologicalHabitatIndex = r
        else:
            self.__biologicalHabitatIndex = [BiologicalHabitatIndex(val)]

    @property
    def activity(self) -> List[Activity]:
        return self.__activity

    @activity.setter
    def activity(self, val: Union[Activity, List[Activity]]) -> None:
        if val is None:
            self.__activity = []
        elif isinstance(val, list):
            r: List[Activity] = []
            for x in val:
                r.append(Activity(x))
            self.__activity = r
        else:
            self.__activity = [Activity(val)]

    @property
    def activityGroup(self) -> List[ActivityGroup]:
        return self.__activityGroup

    @activityGroup.setter
    def activityGroup(self, val: Union[ActivityGroup, List[ActivityGroup]]) -> None:
        if val is None:
            self.__activityGroup = []
        elif isinstance(val, list):
            r: List[ActivityGroup] = []
            for x in val:
                r.append(ActivityGroup(x))
            self.__activityGroup = r
        else:
            self.__activityGroup = [ActivityGroup(val)]

    def generateXML(self, name: str = "Organization") -> str:  # noqa: C901
        doc = Doc()
        asis = doc.asis
        tag = doc.tag

        with tag(name):
            if self.__organizationDescription is None:
                raise WQXException("Attribute 'organizationDescription' is required.")
            asis(self.__organizationDescription.generateXML("OrganizationDescription"))
            for x in self.__electronicAddress:
                asis(x.generateXML("ElectronicAddress"))
            for x in self.__telephonic:
                asis(x.generateXML("Telephonic"))
            if len(self.__organizationAddress) > 3:
                raise WQXException(
                    "Attribute 'organizationAddress' must contain 0 to 3 "
                    "OrganizationAddress objects."
                )
            for x in self.__organizationAddress:
                asis(x.generateXML("OrganizationAddress"))
            for x in self.__project:
                asis(x.generateXML("Project"))
            for x in self.__monitoringLocation:
                asis(x.generateXML("MonitoringLocation"))
            for x in self.__biologicalHabitatIndex:
                asis(x.generateXML("BiologicalHabitatIndex"))
            for x in self.__activity:
                asis(x.generateXML("Activity"))
            for x in self.__activityGroup:
                asis(x.generateXML("ActivityGroup"))

        return doc.getvalue()



from typing import List

from .exceptions import WQXLibException
from .wqx_v3_0 import (
    Activity,
    ActivityDescription,
    ReferenceMethod,
    Result,
    ResultDescription,
    ResultLabInformation,
    SampleDescription,
)
from .WQXResult import WQXResult
from .WQXSample import WQXSample


class WQXActivity(Activity, ActivityDescription):
    __samples: List[WQXSample] = []
    __results: List[WQXResult] = []

    def __init__(self,):
        Activity.__init__(self)
        ActivityDescription.__init__(self)
        self.samples = []
        self.results = []

    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, exception_traceback):
        id = f"{self.monitoringLocationIdentifier}:{self.activityStartDate}:{self.activityTypeCode}"  # noqa B950
        if exception_value is None:
            if self.activityIdentifier is None:
                # Give the activity a default activityIdentifier
                self.activityIdentifier = id
            self.activityDescription = ActivityDescription(self)
            if len(self.__samples) != 1:
                assert WQXLibException(
                    "Each Activity requires a single sampleDescription."
                )
            else:
                sample = self.__samples[0]
                sample.sampleCollectionMethod = ReferenceMethod(sample)
                self.sampleDescription = SampleDescription(sample)
            tmp = []
            for result in self.__results:
                result.resultDescription = ResultDescription(result)
                result.resultLabInformation = ResultLabInformation(result)
                tmp.append(Result(result))
            self.results = tmp

    def sample(self) -> WQXSample:
        tmp = WQXSample()
        self.__samples.append(tmp)
        return tmp

    def result(self) -> WQXResult:
        tmp = WQXResult()
        self.__results.append(tmp)
        return tmp



from yattag import Doc

from ..exceptions import WQXException
from .MeasureCompact import MeasureCompact
from .SimpleContent import (
    ActivityLocationDescriptionText,
    HorizontalCollectionMethodName,
    HorizontalCoordinateReferenceSystemDatumName,
    LatitudeMeasure,
    LongitudeMeasure,
    SourceMapScale,
)


class ActivityLocation:
    """
    Geospatial description of monitoring site, if it is different from that described in
    the station description.
    """

    __latitudeMeasure: LatitudeMeasure
    __longitudeMeasure: LongitudeMeasure
    __sourceMapScale: SourceMapScale
    __horizontalAccuracyMeasure: MeasureCompact
    __horizontalCollectionMethodName: HorizontalCollectionMethodName
    __horizontalCoordinateReferenceSystemDatumName: HorizontalCoordinateReferenceSystemDatumName
    __activityLocationDescriptionText: ActivityLocationDescriptionText

    def __init__(
        self,
        o: dict = None,
        *,
        latitudeMeasure: LatitudeMeasure = None,
        longitudeMeasure: LongitudeMeasure = None,
        sourceMapScale: SourceMapScale = None,
        horizontalAccuracyMeasure: MeasureCompact = None,
        horizontalCollectionMethodName: HorizontalCollectionMethodName = None,
        horizontalCoordinateReferenceSystemDatumName: HorizontalCoordinateReferenceSystemDatumName = None,  # noqa: B950
        activityLocationDescriptionText: ActivityLocationDescriptionText = None
    ):
        if isinstance(o, ActivityLocation):
            # Assign attributes from object without typechecking
            self.__latitudeMeasure = o.latitudeMeasure
            self.__longitudeMeasure = o.longitudeMeasure
            self.__sourceMapScale = o.sourceMapScale
            self.__horizontalAccuracyMeasure = o.horizontalAccuracyMeasure
            self.__horizontalCollectionMethodName = o.horizontalCollectionMethodName
            self.__horizontalCoordinateReferenceSystemDatumName = (
                o.horizontalCoordinateReferenceSystemDatumName
            )
            self.__activityLocationDescriptionText = o.activityLocationDescriptionText
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.latitudeMeasure = o.get("latitudeMeasure")
            self.longitudeMeasure = o.get("longitudeMeasure")
            self.sourceMapScale = o.get("sourceMapScale")
            self.horizontalAccuracyMeasure = o.get("horizontalAccuracyMeasure")
            self.horizontalCollectionMethodName = o.get("horizontalCollectionMethodName")
            self.horizontalCoordinateReferenceSystemDatumName = o.get(
                "horizontalCoordinateReferenceSystemDatumName"
            )
            self.activityLocationDescriptionText = o.get(
                "activityLocationDescriptionText"
            )
        else:
            # Assign attributes from named keywords with typechecking
            self.latitudeMeasure = latitudeMeasure
            self.longitudeMeasure = longitudeMeasure
            self.sourceMapScale = sourceMapScale
            self.horizontalAccuracyMeasure = horizontalAccuracyMeasure
            self.horizontalCollectionMethodName = horizontalCollectionMethodName
            self.horizontalCoordinateReferenceSystemDatumName = (
                horizontalCoordinateReferenceSystemDatumName
            )
            self.activityLocationDescriptionText = activityLocationDescriptionText

    @property
    def latitudeMeasure(self) -> LatitudeMeasure:
        return self.__latitudeMeasure

    @latitudeMeasure.setter
    def latitudeMeasure(self, val: LatitudeMeasure) -> None:
        self.__latitudeMeasure = LatitudeMeasure(val)

    @property
    def longitudeMeasure(self) -> LongitudeMeasure:
        return self.__longitudeMeasure

    @longitudeMeasure.setter
    def longitudeMeasure(self, val: LongitudeMeasure) -> None:
        self.__longitudeMeasure = LongitudeMeasure(val)

    @property
    def sourceMapScale(self) -> SourceMapScale:
        return self.__sourceMapScale

    @sourceMapScale.setter
    def sourceMapScale(self, val: SourceMapScale) -> None:
        self.__sourceMapScale = None if val is None else SourceMapScale(val)

    @property
    def horizontalAccuracyMeasure(self) -> MeasureCompact:
        return self.__horizontalAccuracyMeasure

    @horizontalAccuracyMeasure.setter
    def horizontalAccuracyMeasure(self, val: MeasureCompact) -> None:
        self.__horizontalAccuracyMeasure = None if val is None else MeasureCompact(val)

    @property
    def horizontalCollectionMethodName(self) -> HorizontalCollectionMethodName:
        return self.__horizontalCollectionMethodName

    @horizontalCollectionMethodName.setter
    def horizontalCollectionMethodName(self, val: HorizontalCollectionMethodName) -> None:
        self.__horizontalCollectionMethodName = HorizontalCollectionMethodName(val)

    @property
    def horizontalCoordinateReferenceSystemDatumName(
        self,
    ) -> HorizontalCoordinateReferenceSystemDatumName:
        return self.__horizontalCoordinateReferenceSystemDatumName

    @horizontalCoordinateReferenceSystemDatumName.setter
    def horizontalCoordinateReferenceSystemDatumName(
        self, val: HorizontalCoordinateReferenceSystemDatumName
    ) -> None:
        self.__horizontalCoordinateReferenceSystemDatumName = HorizontalCoordinateReferenceSystemDatumName(  # noqa: B950
            val
        )

    @property
    def activityLocationDescriptionText(self) -> ActivityLocationDescriptionText:
        return self.__activityLocationDescriptionText

    @activityLocationDescriptionText.setter
    def activityLocationDescriptionText(
        self, val: ActivityLocationDescriptionText
    ) -> None:
        self.__activityLocationDescriptionText = (
            None if val is None else ActivityLocationDescriptionText(val)
        )

    def generateXML(self, name: str = "ActivityLocation") -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__latitudeMeasure is None:
                raise WQXException("Attribute 'latitudeMeasure' is required.")
            line("LatitudeMeasure", self.__latitudeMeasure)
            if self.__longitudeMeasure is None:
                raise WQXException("Attribute 'longitudeMeasure' is required.")
            line("LongitudeMeasure", self.__longitudeMeasure)
            if self.__sourceMapScale is not None:
                line("SourceMapScale", self.__sourceMapScale)
            if self.__horizontalAccuracyMeasure is not None:
                asis(
                    self.__horizontalAccuracyMeasure.generateXML(
                        "HorizontalAccuracyMeasure"
                    )
                )
            if self.__horizontalCollectionMethodName is None:
                raise WQXException(
                    "Attribute 'horizontalCollectionMethodName' is required."
                )
            line("HorizontalCollectionMethodName", self.__horizontalCollectionMethodName)
            if self.__horizontalCoordinateReferenceSystemDatumName is None:
                raise WQXException(
                    "Attribute 'horizontalCoordinateReferenceSystemDatumName' is required."
                )
            line(
                "HorizontalCoordinateReferenceSystemDatumName",
                self.__horizontalCoordinateReferenceSystemDatumName,
            )
            if self.__activityLocationDescriptionText is not None:
                line(
                    "ActivityLocationDescriptionText",
                    self.__activityLocationDescriptionText,
                )

        return doc.getvalue()



from yattag import Doc

from ..exceptions import WQXException
from .SimpleContent import (
    MethodIdentifier,
    MethodIdentifierContext,
    MethodModificationText,
)


class ComparableAnalyticalMethod:
    """
    Identifies the procedures, processes, and references required to determine the
    analytical methods used to obtain a result.
    """

    __methodIdentifier: MethodIdentifier
    __methodIdentifierContext: MethodIdentifierContext
    __methodModificationText: MethodModificationText

    def __init__(
        self,
        o: dict = None,
        *,
        methodIdentifier: MethodIdentifier = None,
        methodIdentifierContext: MethodIdentifierContext = None,
        methodModificationText: MethodModificationText = None
    ):
        if isinstance(o, ComparableAnalyticalMethod):
            # Assign attributes from object without typechecking
            self.__methodIdentifier = o.methodIdentifier
            self.__methodIdentifierContext = o.methodIdentifierContext
            self.__methodModificationText = o.methodModificationText
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.methodIdentifier = o.get("methodIdentifier")
            self.methodIdentifierContext = o.get("methodIdentifierContext")
            self.methodModificationText = o.get("methodModificationText")
        else:
            # Assign attributes from named keywords with typechecking
            self.methodIdentifier = methodIdentifier
            self.methodIdentifierContext = methodIdentifierContext
            self.methodModificationText = methodModificationText

    @property
    def methodIdentifier(self) -> MethodIdentifier:
        return self.__methodIdentifier

    @methodIdentifier.setter
    def methodIdentifier(self, val: MethodIdentifier) -> None:
        self.__methodIdentifier = MethodIdentifier(val)

    @property
    def methodIdentifierContext(self) -> MethodIdentifierContext:
        return self.__methodIdentifierContext

    @methodIdentifierContext.setter
    def methodIdentifierContext(self, val: MethodIdentifierContext) -> None:
        self.__methodIdentifierContext = MethodIdentifierContext(val)

    @property
    def methodModificationText(self) -> MethodModificationText:
        return self.__methodModificationText

    @methodModificationText.setter
    def methodModificationText(self, val: MethodModificationText) -> None:
        self.__methodModificationText = (
            None if val is None else MethodModificationText(val)
        )

    def generateXML(self, name: str = "ComparableAnalyticalMethod") -> str:
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__methodIdentifier is None:
                raise WQXException("Attribute 'methodIdentifier' is required.")
            line("MethodIdentifier", self.__methodIdentifier)
            if self.__methodIdentifierContext is None:
                raise WQXException("Attribute 'methodIdentifierContext' is required.")
            line("MethodIdentifierContext", self.__methodIdentifierContext)
            if self.__methodModificationText is not None:
                line("MethodModificationText", self.__methodModificationText)

        return doc.getvalue()



from yattag import Doc

from .SimpleContent import (
    BiasValue,
    ConfidenceIntervalValue,
    LowerConfidenceLimitValue,
    PrecisionValue,
    UpperConfidenceLimitValue,
)


class DataQuality:
    """
    The quantitative statistics and qualitative descriptors that are used to interpret
    the degree of acceptability or utility of data to the user.
    """

    __precisionValue: PrecisionValue  # optional
    __biasValue: BiasValue  # optional
    __confidenceIntervalValue: ConfidenceIntervalValue  # optional
    __upperConfidenceLimitValue: UpperConfidenceLimitValue  # optional
    __lowerConfidenceLimitValue: LowerConfidenceLimitValue  # optional

    def __init__(
        self,
        o: dict = None,
        *,
        precisionValue: PrecisionValue = None,
        biasValue: BiasValue = None,
        confidenceIntervalValue: ConfidenceIntervalValue = None,
        upperConfidenceLimitValue: UpperConfidenceLimitValue = None,
        lowerConfidenceLimitValue: LowerConfidenceLimitValue = None
    ):
        if isinstance(o, DataQuality):
            # Assign attributes from object without typechecking
            self.__precisionValue = o.precisionValue
            self.__biasValue = o.biasValue
            self.__confidenceIntervalValue = o.confidenceIntervalValue
            self.__upperConfidenceLimitValue = o.upperConfidenceLimitValue
            self.__lowerConfidenceLimitValue = o.lowerConfidenceLimitValue
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.precisionValue = o.get("precisionValue")
            self.biasValue = o.get("biasValue")
            self.confidenceIntervalValue = o.get("confidenceIntervalValue")
            self.upperConfidenceLimitValue = o.get("upperConfidenceLimitValue")
            self.lowerConfidenceLimitValue = o.get("lowerConfidenceLimitValue")
        else:
            # Assign attributes from named keywords with typechecking
            self.precisionValue = precisionValue
            self.biasValue = biasValue
            self.confidenceIntervalValue = confidenceIntervalValue
            self.upperConfidenceLimitValue = upperConfidenceLimitValue
            self.lowerConfidenceLimitValue = lowerConfidenceLimitValue

    @property
    def precisionValue(self) -> PrecisionValue:
        return self.__precisionValue

    @precisionValue.setter
    def precisionValue(self, val: PrecisionValue) -> None:
        self.__precisionValue = None if val is None else PrecisionValue(val)

    @property
    def biasValue(self) -> BiasValue:
        return self.__biasValue

    @biasValue.setter
    def biasValue(self, val: BiasValue) -> None:
        self.__biasValue = None if val is None else BiasValue(val)

    @property
    def confidenceIntervalValue(self) -> ConfidenceIntervalValue:
        return self.__confidenceIntervalValue

    @confidenceIntervalValue.setter
    def confidenceIntervalValue(self, val: ConfidenceIntervalValue) -> None:
        self.__confidenceIntervalValue = (
            None if val is None else ConfidenceIntervalValue(val)
        )

    @property
    def upperConfidenceLimitValue(self) -> UpperConfidenceLimitValue:
        return self.__upperConfidenceLimitValue

    @upperConfidenceLimitValue.setter
    def upperConfidenceLimitValue(self, val: UpperConfidenceLimitValue) -> None:
        self.__upperConfidenceLimitValue = (
            None if val is None else UpperConfidenceLimitValue(val)
        )

    @property
    def lowerConfidenceLimitValue(self) -> LowerConfidenceLimitValue:
        return self.__lowerConfidenceLimitValue

    @lowerConfidenceLimitValue.setter
    def lowerConfidenceLimitValue(self, val: LowerConfidenceLimitValue) -> None:
        self.__lowerConfidenceLimitValue = (
            None if val is None else LowerConfidenceLimitValue(val)
        )

    def generateXML(self, name: str = "DataQuality") -> str:
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__precisionValue is not None:
                line("PrecisionValue", self.__precisionValue)
            if self.__biasValue is not None:
                line("BiasValue", self.__biasValue)
            if self.__confidenceIntervalValue is not None:
                line("ConfidenceIntervalValue", self.__confidenceIntervalValue)
            if self.__upperConfidenceLimitValue is not None:
                line("UpperConfidenceLimitValue", self.__upperConfidenceLimitValue)
            if self.__lowerConfidenceLimitValue is not None:
                line("LowerConfidenceLimitValue", self.__lowerConfidenceLimitValue)

        return doc.getvalue()



from yattag import Doc

from .ReferenceMethod import ReferenceMethod
from .SimpleContent import (
    ChemicalPreservativeUsedName,
    SampleContainerColorName,
    SampleContainerLabelName,
    SampleContainerTypeName,
    SampleTransportStorageDescription,
    ThermalPreservativeUsedName,
)


class SamplePreparation:
    """
    Describes a sample preparation procedure which may be conducted on an initial Sample
    or on subsequent subsamples.
    """

    __samplePreparationMethod: ReferenceMethod
    __sampleContainerLabelName: SampleContainerLabelName
    __sampleContainerTypeName: SampleContainerTypeName
    __sampleContainerColorName: SampleContainerColorName
    __chemicalPreservativeUsedName: ChemicalPreservativeUsedName
    __thermalPreservativeUsedName: ThermalPreservativeUsedName
    __sampleTransportStorageDescription: SampleTransportStorageDescription

    def __init__(
        self,
        o: dict = None,
        *,
        samplePreparationMethod: ReferenceMethod = None,
        sampleContainerLabelName: SampleContainerLabelName = None,
        sampleContainerTypeName: SampleContainerTypeName = None,
        sampleContainerColorName: SampleContainerColorName = None,
        chemicalPreservativeUsedName: ChemicalPreservativeUsedName = None,
        thermalPreservativeUsedName: ThermalPreservativeUsedName = None,
        sampleTransportStorageDescription: SampleTransportStorageDescription = None
    ):
        if isinstance(o, SamplePreparation):
            # Assign attributes from object without typechecking
            self.__samplePreparationMethod = o.samplePreparationMethod
            self.__sampleContainerLabelName = o.sampleContainerLabelName
            self.__sampleContainerTypeName = o.sampleContainerTypeName
            self.__sampleContainerColorName = o.sampleContainerColorName
            self.__chemicalPreservativeUsedName = o.chemicalPreservativeUsedName
            self.__thermalPreservativeUsedName = o.thermalPreservativeUsedName
            self.__sampleTransportStorageDescription = o.sampleTransportStorageDescription
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.samplePreparationMethod = o.get("samplePreparationMethod")
            self.sampleContainerLabelName = o.get("sampleContainerLabelName")
            self.sampleContainerTypeName = o.get("sampleContainerTypeName")
            self.sampleContainerColorName = o.get("sampleContainerColorName")
            self.chemicalPreservativeUsedName = o.get("chemicalPreservativeUsedName")
            self.thermalPreservativeUsedName = o.get("thermalPreservativeUsedName")
            self.sampleTransportStorageDescription = o.get(
                "sampleTransportStorageDescription"
            )
        else:
            # Assign attributes from named keywords with typechecking
            self.samplePreparationMethod = samplePreparationMethod
            self.sampleContainerLabelName = sampleContainerLabelName
            self.sampleContainerTypeName = sampleContainerTypeName
            self.sampleContainerColorName = sampleContainerColorName
            self.chemicalPreservativeUsedName = chemicalPreservativeUsedName
            self.thermalPreservativeUsedName = thermalPreservativeUsedName
            self.sampleTransportStorageDescription = sampleTransportStorageDescription

    @property
    def samplePreparationMethod(self) -> ReferenceMethod:
        """
        Identifying information about the method(s) followed to prepare a sample for
        analysis.
        """
        return self.__samplePreparationMethod

    @samplePreparationMethod.setter
    def samplePreparationMethod(self, val: ReferenceMethod) -> None:
        """
        Identifying information about the method(s) followed to prepare a sample for
        analysis.
        """
        self.__samplePreparationMethod = None if val is None else ReferenceMethod(val)

    @property
    def sampleContainerLabelName(self) -> SampleContainerLabelName:
        return self.__sampleContainerLabelName

    @sampleContainerLabelName.setter
    def sampleContainerLabelName(self, val: SampleContainerLabelName) -> None:
        self.__sampleContainerLabelName = (
            None if val is None else SampleContainerLabelName(val)
        )

    @property
    def sampleContainerTypeName(self) -> SampleContainerTypeName:
        return self.__sampleContainerTypeName

    @sampleContainerTypeName.setter
    def sampleContainerTypeName(self, val: SampleContainerTypeName) -> None:
        self.__sampleContainerTypeName = (
            None if val is None else SampleContainerTypeName(val)
        )

    @property
    def sampleContainerColorName(self) -> SampleContainerColorName:
        return self.__sampleContainerColorName

    @sampleContainerColorName.setter
    def sampleContainerColorName(self, val: SampleContainerColorName) -> None:
        self.__sampleContainerColorName = (
            None if val is None else SampleContainerColorName(val)
        )

    @property
    def chemicalPreservativeUsedName(self) -> ChemicalPreservativeUsedName:
        return self.__chemicalPreservativeUsedName

    @chemicalPreservativeUsedName.setter
    def chemicalPreservativeUsedName(self, val: ChemicalPreservativeUsedName) -> None:
        self.__chemicalPreservativeUsedName = (
            None if val is None else ChemicalPreservativeUsedName(val)
        )

    @property
    def thermalPreservativeUsedName(self) -> ThermalPreservativeUsedName:
        return self.__thermalPreservativeUsedName

    @thermalPreservativeUsedName.setter
    def thermalPreservativeUsedName(self, val: ThermalPreservativeUsedName) -> None:
        self.__thermalPreservativeUsedName = (
            None if val is None else ThermalPreservativeUsedName(val)
        )

    @property
    def sampleTransportStorageDescription(self) -> SampleTransportStorageDescription:
        return self.__sampleTransportStorageDescription

    @sampleTransportStorageDescription.setter
    def sampleTransportStorageDescription(
        self, val: SampleTransportStorageDescription
    ) -> None:
        self.__sampleTransportStorageDescription = (
            None if val is None else SampleTransportStorageDescription(val)
        )

    def generateXML(self, name: str = "SamplePreparation") -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__samplePreparationMethod is not None:
                asis(
                    self.__samplePreparationMethod.generateXML("SamplePreparationMethod")
                )
            if self.__sampleContainerLabelName is not None:
                line("SampleContainerLabelName", self.__sampleContainerLabelName)
            if self.__sampleContainerTypeName is not None:
                line("SampleContainerTypeName", self.__sampleContainerTypeName)
            if self.__sampleContainerColorName is not None:
                line("SampleContainerColorName", self.__sampleContainerColorName)
            if self.__chemicalPreservativeUsedName is not None:
                line("ChemicalPreservativeUsedName", self.__chemicalPreservativeUsedName)
            if self.__thermalPreservativeUsedName is not None:
                line("ThermalPreservativeUsedName", self.__thermalPreservativeUsedName)
            if self.__sampleTransportStorageDescription is not None:
                line(
                    "SampleTransportStorageDescription",
                    self.__sampleTransportStorageDescription,
                )

        return doc.getvalue()



from typing import List, Union

from yattag import Doc

from ..exceptions import WQXException
from .AttachedBinaryObject import AttachedBinaryObject
from .ProjectMonitoringLocationWeighting import ProjectMonitoringLocationWeighting
from .SimpleContent import (
    ProjectDescriptionText,
    ProjectIdentifier,
    ProjectName,
    QAPPApprovalAgencyName,
    QAPPApprovedIndicator,
    SamplingDesignTypeCode,
)


class Project:
    """
    An environmental data collection effort that has a stated purpose and puts a series
    of samples and results into a meaningful context.
    """

    __projectIdentifier: ProjectIdentifier
    __projectName: ProjectName
    __projectDescriptionText: ProjectDescriptionText
    __samplingDesignTypeCode: SamplingDesignTypeCode
    __qAPPApprovedIndicator: QAPPApprovedIndicator
    __qAPPApprovalAgencyName: QAPPApprovalAgencyName
    __attachedBinaryObject: List[AttachedBinaryObject]
    __projectMonitoringLocationWeighting: List[ProjectMonitoringLocationWeighting]

    def __init__(
        self,
        o: dict = None,
        *,
        projectIdentifier: ProjectIdentifier = None,
        projectName: ProjectName = None,
        projectDescriptionText: ProjectDescriptionText = None,
        samplingDesignTypeCode: SamplingDesignTypeCode = None,
        qAPPApprovedIndicator: QAPPApprovedIndicator = None,
        qAPPApprovalAgencyName: QAPPApprovalAgencyName = None,
        attachedBinaryObject: List[AttachedBinaryObject] = None,
        projectMonitoringLocationWeighting: List[
            ProjectMonitoringLocationWeighting
        ] = None
    ):
        if isinstance(o, Project):
            # Assign attributes from object without typechecking
            self.__projectIdentifier = o.projectIdentifier
            self.__projectName = o.projectName
            self.__projectDescriptionText = o.projectDescriptionText
            self.__samplingDesignTypeCode = o.samplingDesignTypeCode
            self.__qAPPApprovedIndicator = o.qAPPApprovedIndicator
            self.__qAPPApprovalAgencyName = o.qAPPApprovalAgencyName
            self.__attachedBinaryObject = o.attachedBinaryObject
            self.__projectMonitoringLocationWeighting = (
                o.projectMonitoringLocationWeighting
            )
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.projectIdentifier = o.get("projectIdentifier")
            self.projectName = o.get("projectName")
            self.projectDescriptionText = o.get("projectDescriptionText")
            self.samplingDesignTypeCode = o.get("samplingDesignTypeCode")
            self.qAPPApprovedIndicator = o.get("qAPPApprovedIndicator")
            self.qAPPApprovalAgencyName = o.get("qAPPApprovalAgencyName")
            self.attachedBinaryObject = o.get("attachedBinaryObject")
            self.projectMonitoringLocationWeighting = o.get(
                "projectMonitoringLocationWeighting"
            )
        else:
            # Assign attributes from named keywords with typechecking
            self.projectIdentifier = projectIdentifier
            self.projectName = projectName
            self.projectDescriptionText = projectDescriptionText
            self.samplingDesignTypeCode = samplingDesignTypeCode
            self.qAPPApprovedIndicator = qAPPApprovedIndicator
            self.qAPPApprovalAgencyName = qAPPApprovalAgencyName
            self.attachedBinaryObject = attachedBinaryObject
            self.projectMonitoringLocationWeighting = projectMonitoringLocationWeighting

    @property
    def projectIdentifier(self) -> ProjectIdentifier:
        return self.__projectIdentifier

    @projectIdentifier.setter
    def projectIdentifier(self, val: ProjectIdentifier) -> None:
        self.__projectIdentifier = ProjectIdentifier(val)

    @property
    def projectName(self) -> ProjectName:
        return self.__projectName

    @projectName.setter
    def projectName(self, val: ProjectName) -> None:
        self.__projectName = ProjectName(val)

    @property
    def projectDescriptionText(self) -> ProjectDescriptionText:
        return self.__projectDescriptionText

    @projectDescriptionText.setter
    def projectDescriptionText(self, val: ProjectDescriptionText) -> None:
        self.__projectDescriptionText = (
            None if val is None else ProjectDescriptionText(val)
        )

    @property
    def samplingDesignTypeCode(self) -> SamplingDesignTypeCode:
        return self.__samplingDesignTypeCode

    @samplingDesignTypeCode.setter
    def samplingDesignTypeCode(self, val: SamplingDesignTypeCode) -> None:
        self.__samplingDesignTypeCode = (
            None if val is None else SamplingDesignTypeCode(val)
        )

    @property
    def qAPPApprovedIndicator(self) -> QAPPApprovedIndicator:
        return self.__qAPPApprovedIndicator

    @qAPPApprovedIndicator.setter
    def qAPPApprovedIndicator(self, val: QAPPApprovedIndicator) -> None:
        self.__qAPPApprovedIndicator = None if val is None else QAPPApprovedIndicator(val)

    @property
    def qAPPApprovalAgencyName(self) -> QAPPApprovalAgencyName:
        return self.__qAPPApprovalAgencyName

    @qAPPApprovalAgencyName.setter
    def qAPPApprovalAgencyName(self, val: QAPPApprovalAgencyName) -> None:
        self.__qAPPApprovalAgencyName = (
            None if val is None else QAPPApprovalAgencyName(val)
        )

    @property
    def attachedBinaryObject(self) -> List[AttachedBinaryObject]:
        return self.__attachedBinaryObject

    @attachedBinaryObject.setter
    def attachedBinaryObject(
        self, val: Union[AttachedBinaryObject, List[AttachedBinaryObject]]
    ) -> None:
        if val is None:
            self.__attachedBinaryObject = []
        elif isinstance(val, list):
            r: List[AttachedBinaryObject] = []
            for x in val:
                r.append(AttachedBinaryObject(x))
            self.__attachedBinaryObject = r
        else:
            self.__attachedBinaryObject = [AttachedBinaryObject(val)]

    @property
    def projectMonitoringLocationWeighting(
        self,
    ) -> List[ProjectMonitoringLocationWeighting]:
        return self.__projectMonitoringLocationWeighting

    @projectMonitoringLocationWeighting.setter
    def projectMonitoringLocationWeighting(
        self,
        val: Union[
            ProjectMonitoringLocationWeighting, List[ProjectMonitoringLocationWeighting]
        ],
    ) -> None:
        if val is None:
            self.__projectMonitoringLocationWeighting = []
        elif isinstance(val, list):
            r: List[ProjectMonitoringLocationWeighting] = []
            for x in val:
                r.append(ProjectMonitoringLocationWeighting(x))
            self.__projectMonitoringLocationWeighting = r
        else:
            self.__projectMonitoringLocationWeighting = [
                ProjectMonitoringLocationWeighting(val)
            ]

    def generateXML(self, name: str = "Project") -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__projectIdentifier is None:
                raise WQXException("Attribute 'projectIdentifier' is required.")
            line("ProjectIdentifier", self.__projectIdentifier)
            if self.__projectName is None:
                raise WQXException("Attribute 'projectName' is required.")
            line("ProjectName", self.__projectName)
            if self.__projectDescriptionText is not None:
                line("ProjectDescriptionText", self.__projectDescriptionText)
            if self.__samplingDesignTypeCode is not None:
                line("SamplingDesignTypeCode", self.__samplingDesignTypeCode)
            if self.__qAPPApprovedIndicator is not None:
                line("QAPPApprovedIndicator", self.__qAPPApprovedIndicator)
            if self.__qAPPApprovalAgencyName is not None:
                line("QAPPApprovalAgencyName", self.__qAPPApprovalAgencyName)
            for x in self.__attachedBinaryObject:
                asis(x.generateXML("AttachedBinaryObject"))
            for x in self.__projectMonitoringLocationWeighting:
                asis(x.generateXML("ProjectMonitoringLocationWeighting"))

        return doc.getvalue()



from yattag import Doc

from ..exceptions import WQXException
from .AquiferInformation import AquiferInformation
from .MeasureCompact import MeasureCompact
from .SimpleContent import (
    AquiferTypeName,
    ConstructionDate,
    FormationTypeText,
    NationalAquiferCode,
    WellTypeText,
)


class WellInformation:
    """
    Description of the attributes of a well.
    """

    __wellTypeText: WellTypeText = None
    __aquiferTypeName: AquiferTypeName = None
    __nationalAquiferCode: NationalAquiferCode = None
    __aquiferInformation: AquiferInformation = None
    __formationTypeText: FormationTypeText = None
    __wellHoleDepthMeasure: MeasureCompact = None
    __constructionDate: ConstructionDate = None
    __wellDepthMeasure: MeasureCompact = None

    def __init__(
        self,
        o: dict = None,
        *,
        wellTypeText: WellTypeText = None,
        aquiferTypeName: AquiferTypeName = None,
        nationalAquiferCode: NationalAquiferCode = None,
        aquiferInformation: AquiferInformation = None,
        formationTypeText: FormationTypeText = None,
        wellHoleDepthMeasure: MeasureCompact = None,
        constructionDate: ConstructionDate = None,
        wellDepthMeasure: MeasureCompact = None
    ):
        if isinstance(o, WellInformation):
            # Assign attributes from object without typechecking
            self.__wellTypeText = o.wellTypeText
            self.__aquiferTypeName = o.aquiferTypeName
            self.__nationalAquiferCode = o.nationalAquiferCode
            self.__aquiferInformation = o.aquiferInformation
            self.__formationTypeText = o.formationTypeText
            self.__wellHoleDepthMeasure = o.wellHoleDepthMeasure
            self.__constructionDate = o.constructionDate
            self.__wellDepthMeasure = o.wellDepthMeasure
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.wellTypeText = o.get("wellTypeText")
            self.aquiferTypeName = o.get("aquiferTypeName")
            self.nationalAquiferCode = o.get("nationalAquiferCode")
            self.aquiferInformation = o.get("aquiferInformation")
            self.formationTypeText = o.get("formationTypeText")
            self.wellHoleDepthMeasure = o.get("wellHoleDepthMeasure")
            self.constructionDate = o.get("constructionDate")
            self.wellDepthMeasure = o.get("wellDepthMeasure")
        else:
            # Assign attributes from named keywords with typechecking
            self.wellTypeText = wellTypeText
            self.aquiferTypeName = aquiferTypeName
            self.nationalAquiferCode = nationalAquiferCode
            self.aquiferInformation = aquiferInformation
            self.formationTypeText = formationTypeText
            self.wellHoleDepthMeasure = wellHoleDepthMeasure
            self.constructionDate = constructionDate
            self.wellDepthMeasure = wellDepthMeasure

    @property
    def wellTypeText(self) -> WellTypeText:
        return self.__wellTypeText

    @wellTypeText.setter
    def wellTypeText(self, val: WellTypeText) -> None:
        self.__wellTypeText = None if val is None else WellTypeText(val)

    @property
    def aquiferTypeName(self) -> AquiferTypeName:
        return self.__aquiferTypeName

    @aquiferTypeName.setter
    def aquiferTypeName(self, val: AquiferTypeName) -> None:
        self.__aquiferTypeName = None if val is None else AquiferTypeName(val)

    @property
    def nationalAquiferCode(self) -> NationalAquiferCode:
        return self.__nationalAquiferCode

    @nationalAquiferCode.setter
    def nationalAquiferCode(self, val: NationalAquiferCode) -> None:
        self.__nationalAquiferCode = None if val is None else NationalAquiferCode(val)

    @property
    def aquiferInformation(self) -> AquiferInformation:
        return self.__aquiferInformation

    @aquiferInformation.setter
    def aquiferInformation(self, val: AquiferInformation) -> None:
        self.__aquiferInformation = None if val is None else AquiferInformation(val)

    @property
    def formationTypeText(self) -> FormationTypeText:
        return self.__formationTypeText

    @formationTypeText.setter
    def formationTypeText(self, val: FormationTypeText) -> None:
        self.__formationTypeText = None if val is None else FormationTypeText(val)

    @property
    def wellHoleDepthMeasure(self) -> MeasureCompact:
        """
        Depth below land surface datum (LSD) to the bottom of the hole on completion of
        drilling.
        """
        return self.__wellHoleDepthMeasure

    @wellHoleDepthMeasure.setter
    def wellHoleDepthMeasure(self, val: MeasureCompact) -> None:
        """
        Depth below land surface datum (LSD) to the bottom of the hole on completion of
        drilling.
        """
        self.__wellHoleDepthMeasure = None if val is None else MeasureCompact(val)

    @property
    def constructionDate(self) -> ConstructionDate:
        return self.__constructionDate

    @constructionDate.setter
    def constructionDate(self, val: ConstructionDate) -> None:
        self.__constructionDate = None if val is None else ConstructionDate(val)

    @property
    def wellDepthMeasure(self) -> MeasureCompact:
        """
        Depth below land surface datum (LSD) to the bottom of the hole on completion of
        drilling. ie. completion depth
        """
        return self.__wellDepthMeasure

    @wellDepthMeasure.setter
    def wellDepthMeasure(self, val: MeasureCompact) -> None:
        """
        Depth below land surface datum (LSD) to the bottom of the hole on completion of
        drilling. ie. completion depth
        """
        self.__wellDepthMeasure = None if val is None else MeasureCompact(val)

    def generateXML(self, name: str = "WellInformation") -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__wellTypeText is None:
                raise WQXException("Attribute 'wellTypeText' is required.")
            line("WellTypeText", self.__wellTypeText)
            if self.__aquiferTypeName is not None:
                line("AquiferTypeName", self.__aquiferTypeName)
            if self.__nationalAquiferCode is not None:
                line("NationalAquiferCode", self.__nationalAquiferCode)
            if self.__aquiferInformation is not None:
                asis(self.__aquiferInformation.generateXML("AquiferInformation"))
            if self.__formationTypeText is not None:
                line("FormationTypeText", self.__formationTypeText)
            if self.__wellHoleDepthMeasure is not None:
                asis(self.__wellHoleDepthMeasure.generateXML("WellHoleDepthMeasure"))
            if self.__constructionDate is not None:
                line("ConstructionDate", self.__constructionDate)
            if self.__wellDepthMeasure is not None:
                asis(self.__wellDepthMeasure.generateXML("WellDepthMeasure"))

        return doc.getvalue()



from yattag import Doc

from ..exceptions import WQXException
from .Organization import Organization


class WQX:
    """
    Main Schema used to transfer water monitoring results to EPA Office of Water.
    """

    __organization: Organization

    def __init__(self, o: dict = None, *, organization: Organization = None):
        if isinstance(o, WQX):
            # Assign attributes from object without typechecking
            self.__organization = o.organization
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.organization = o.get("organization")
        else:
            # Assign attributes from named keywords with typechecking
            self.organization = organization

    @property
    def organization(self) -> Organization:
        return self.__organization

    @organization.setter
    def organization(self, val: Organization) -> None:
        if val is not None and not isinstance(val, Organization):
            raise WQXException("Attribute 'organization' must be an Organization object.")
        self.__organization = None if val is None else Organization(val)

    def generateXML(self, name="WQX"):
        doc = Doc()
        asis = doc.asis
        tag = doc.tag

        with tag(
            name,
            ("xmlns", "http://www.exchangenetwork.net/schema/wqx/3"),
            ("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance"),
            (
                "xsi:schemaLocation",
                "http://www.exchangenetwork.net/schema/wqx/3 "
                "http://www.exchangenetwork.net/schema/wqx/3/index.xsd",
            ),
        ):
            if self.__organization is None:
                raise WQXException("Attribute 'organization' is required.")
            asis(self.__organization.generateXML("Organization"))

        return doc.getvalue()



from typing import List, Union

from yattag import Doc

from ..exceptions import WQXException
from .SimpleContent import (
    ActivityGroupIdentifier,
    ActivityIdentifier,
    IndexIdentifier,
    MonitoringLocationIdentifier,
    OrganizationIdentifier,
    ProjectIdentifier,
)


class OrganizationDelete:
    """
    Schema used to delete organization information
    """

    __organizationIdentifier: OrganizationIdentifier
    __projectIdentifier: List[ProjectIdentifier]
    __monitoringLocationIdentifier: List[MonitoringLocationIdentifier]
    __activityIdentifier: List[ActivityIdentifier]
    __activityGroupIdentifier: List[ActivityGroupIdentifier]
    __indexIdentifier: List[IndexIdentifier]

    def __init__(
        self,
        o: dict = None,
        *,
        organizationIdentifier: OrganizationIdentifier = None,
        projectIdentifier: List[ProjectIdentifier] = None,
        monitoringLocationIdentifier: List[MonitoringLocationIdentifier] = None,
        activityIdentifier: List[ActivityIdentifier] = None,
        activityGroupIdentifier: List[ActivityGroupIdentifier] = None,
        indexIdentifier: List[IndexIdentifier] = None
    ):
        if isinstance(o, OrganizationDelete):
            # Assign attributes from object without typechecking
            self.__organizationIdentifier = o.organizationIdentifier
            self.__projectIdentifier = o.projectIdentifier
            self.__monitoringLocationIdentifier = o.monitoringLocationIdentifier
            self.__activityIdentifier = o.activityIdentifier
            self.__activityGroupIdentifier = o.activityGroupIdentifier
            self.__indexIdentifier = o.indexIdentifier
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.organizationIdentifier = o.get("organizationIdentifier")
            self.projectIdentifier = o.get("projectIdentifier")
            self.monitoringLocationIdentifier = o.get("monitoringLocationIdentifier")
            self.activityIdentifier = o.get("activityIdentifier")
            self.activityGroupIdentifier = o.get("activityGroupIdentifier")
            self.indexIdentifier = o.get("indexIdentifier")
        else:
            # Assign attributes from named keywords with typechecking
            self.organizationIdentifier = organizationIdentifier
            self.projectIdentifier = projectIdentifier
            self.monitoringLocationIdentifier = monitoringLocationIdentifier
            self.activityIdentifier = activityIdentifier
            self.activityGroupIdentifier = activityGroupIdentifier
            self.indexIdentifier = indexIdentifier

    @property
    def organizationIdentifier(self) -> OrganizationIdentifier:
        return self.__organizationIdentifier

    @organizationIdentifier.setter
    def organizationIdentifier(self, val: OrganizationIdentifier) -> None:
        self.__organizationIdentifier = (
            None if val is None else OrganizationIdentifier(val)
        )

    @property
    def projectIdentifier(self) -> List[ProjectIdentifier]:
        return self.__projectIdentifier

    @projectIdentifier.setter
    def projectIdentifier(
        self, val: Union[ProjectIdentifier, List[ProjectIdentifier]]
    ) -> None:
        if val is None:
            self.__projectIdentifier = []
        elif isinstance(val, list):
            r: List[ProjectIdentifier] = []
            for x in val:
                r.append(ProjectIdentifier(x))
            self.__projectIdentifier = r
        else:
            self.__projectIdentifier = [ProjectIdentifier(val)]

    @property
    def monitoringLocationIdentifier(self) -> List[MonitoringLocationIdentifier]:
        return self.__monitoringLocationIdentifier

    @monitoringLocationIdentifier.setter
    def monitoringLocationIdentifier(
        self,
        val: Union[MonitoringLocationIdentifier, List[MonitoringLocationIdentifier]],
    ) -> None:
        if val is None:
            self.__monitoringLocationIdentifier = []
        elif isinstance(val, list):
            r: List[MonitoringLocationIdentifier] = []
            for x in val:
                r.append(MonitoringLocationIdentifier(x))
            self.__monitoringLocationIdentifier = r
        else:
            self.__monitoringLocationIdentifier = [MonitoringLocationIdentifier(val)]

    @property
    def activityIdentifier(self) -> List[ActivityIdentifier]:
        return self.__activityIdentifier

    @activityIdentifier.setter
    def activityIdentifier(
        self, val: Union[ActivityIdentifier, List[ActivityIdentifier]]
    ) -> None:
        if val is None:
            self.__activityIdentifier = []
        elif isinstance(val, list):
            r: List[ActivityIdentifier] = []
            for x in val:
                r.append(ActivityIdentifier(x))
            self.__activityIdentifier = r
        else:
            self.__activityIdentifier = [ActivityIdentifier(val)]

    @property
    def activityGroupIdentifier(self) -> List[ActivityGroupIdentifier]:
        return self.__activityGroupIdentifier

    @activityGroupIdentifier.setter
    def activityGroupIdentifier(
        self, val: Union[ActivityGroupIdentifier, List[ActivityGroupIdentifier]]
    ) -> None:
        if val is None:
            self.__activityGroupIdentifier = []
        elif isinstance(val, list):
            r: List[ActivityGroupIdentifier] = []
            for x in val:
                r.append(ActivityGroupIdentifier(x))
            self.__activityGroupIdentifier = r
        else:
            self.__activityGroupIdentifier = [ActivityGroupIdentifier(val)]

    @property
    def indexIdentifier(self) -> List[IndexIdentifier]:
        return self.__indexIdentifier

    @indexIdentifier.setter
    def indexIdentifier(self, val: Union[IndexIdentifier, List[IndexIdentifier]]) -> None:
        if val is None:
            self.__indexIdentifier = []
        elif isinstance(val, list):
            r: List[IndexIdentifier] = []
            for x in val:
                r.append(IndexIdentifier(x))
            self.__indexIdentifier = r
        else:
            self.__indexIdentifier = [IndexIdentifier(val)]

    def generateXML(self, name="OrganizationDelete"):
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__organizationIdentifier is None:
                raise WQXException("Attribute 'organizationIdentifier' is required.")
            line("OrganizationIdentifier", self.__organizationIdentifier)
            for x in self.__projectIdentifier:
                line("ProjectIdentifier", x)
            for x in self.__monitoringLocationIdentifier:
                line("MonitoringLocationIdentifier", x)
            for x in self.__activityIdentifier:
                line("ActivityIdentifier", x)
            for x in self.__activityGroupIdentifier:
                line("ActivityGroupIdentifier", x)
            for x in self.__indexIdentifier:
                line("IndexIdentifier", x)

        return doc.getvalue()



from yattag import Doc

from ..exceptions import WQXException
from .MeasureCompact import MeasureCompact
from .SimpleContent import NetTypeName


class NetInformation:
    """
    Allows for the reporting of net sample collection information.
    """

    __netTypeName: NetTypeName
    __netSurfaceAreaMeasure: MeasureCompact
    __netMeshSizeMeasure: MeasureCompact
    __boatSpeedMeasure: MeasureCompact
    __currentSpeedMeasure: MeasureCompact

    def __init__(
        self,
        o: dict = None,
        *,
        netTypeName: NetTypeName = None,
        netSurfaceAreaMeasure: MeasureCompact = None,
        netMeshSizeMeasure: MeasureCompact = None,
        boatSpeedMeasure: MeasureCompact = None,
        currentSpeedMeasure: MeasureCompact = None
    ):
        if isinstance(o, NetInformation):
            # Assign attributes from object without typechecking
            self.__netTypeName = o.netTypeName
            self.__netSurfaceAreaMeasure = o.netSurfaceAreaMeasure
            self.__netMeshSizeMeasure = o.netMeshSizeMeasure
            self.__boatSpeedMeasure = o.boatSpeedMeasure
            self.__currentSpeedMeasure = o.currentSpeedMeasure
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.netTypeName = o.get("netTypeName")
            self.netSurfaceAreaMeasure = o.get("netSurfaceAreaMeasure")
            self.netMeshSizeMeasure = o.get("netMeshSizeMeasure")
            self.boatSpeedMeasure = o.get("boatSpeedMeasure")
            self.currentSpeedMeasure = o.get("currentSpeedMeasure")
        else:
            # Assign attributes from named keywords with typechecking
            self.netTypeName = netTypeName
            self.netSurfaceAreaMeasure = netSurfaceAreaMeasure
            self.netMeshSizeMeasure = netMeshSizeMeasure
            self.boatSpeedMeasure = boatSpeedMeasure
            self.currentSpeedMeasure = currentSpeedMeasure

    @property
    def netTypeName(self) -> NetTypeName:
        return self.__netTypeName

    @netTypeName.setter
    def netTypeName(self, val: NetTypeName) -> None:
        self.__netTypeName = NetTypeName(val)

    @property
    def netSurfaceAreaMeasure(self) -> MeasureCompact:
        """
        A measurement of the effective surface area of the net used during biological
        monitoring sample collection.
        """
        return self.netSurfaceAreaMeasure

    @netSurfaceAreaMeasure.setter
    def netSurfaceAreaMeasure(self, val: MeasureCompact) -> None:
        """
        A measurement of the effective surface area of the net used during biological
        monitoring sample collection.
        """
        self.__netSurfaceAreaMeasure = None if val is None else MeasureCompact(val)

    @property
    def netMeshSizeMeasure(self) -> MeasureCompact:
        """
        A measurement of the mesh size of the net used during biological monitoring
        sample collection.
        """
        return self.netMeshSizeMeasure

    @netMeshSizeMeasure.setter
    def netMeshSizeMeasure(self, val: MeasureCompact) -> None:
        """
        A measurement of the mesh size of the net used during biological monitoring
        sample collection.
        """
        self.__netMeshSizeMeasure = None if val is None else MeasureCompact(val)

    @property
    def boatSpeedMeasure(self) -> MeasureCompact:
        """
        A measurement of the boat speed during biological monitoring sample collection.
        """
        return self.boatSpeedMeasure

    @boatSpeedMeasure.setter
    def boatSpeedMeasure(self, val: MeasureCompact) -> None:
        """
        A measurement of the boat speed during biological monitoring sample collection.
        """
        self.__boatSpeedMeasure = None if val is None else MeasureCompact(val)

    @property
    def currentSpeedMeasure(self) -> MeasureCompact:
        """
        A measurement of the current during biological monitoring sample collection.
        """
        return self.currentSpeedMeasure

    @currentSpeedMeasure.setter
    def currentSpeedMeasure(self, val: MeasureCompact) -> None:
        """
        A measurement of the current during biological monitoring sample collection.
        """
        self.__currentSpeedMeasure = None if val is None else MeasureCompact(val)

    def generateXML(self, name: str = "NetInformation") -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__netTypeName is None:
                raise WQXException("Attribute 'netTypeName' is required.")
            line("NetTypeName", self.__netTypeName)
            if self.__netSurfaceAreaMeasure is not None:
                asis(self.__netSurfaceAreaMeasure.generateXML("NetSurfaceAreaMeasure"))
            if self.__netMeshSizeMeasure is not None:
                asis(self.__netMeshSizeMeasure.generateXML("NetMeshSizeMeasure"))
            if self.__boatSpeedMeasure is not None:
                asis(self.__boatSpeedMeasure.generateXML("BoatSpeedMeasure"))
            if self.__currentSpeedMeasure is not None:
                asis(self.__currentSpeedMeasure.generateXML("CurrentSpeedMeasure"))

        return doc.getvalue()



from typing import List

from .wqx_v3_0 import (
    Measure,
    MeasureCompact,
    MeasureQualifierCode,
    MeasureUnitCode,
    MeasureValue,
    Result,
    ResultDescription,
    ResultLabInformation,
    ResultMeasureValue,
)


class WQXResult(Result, ResultDescription, ResultLabInformation):
    def __init__(self,):
        ResultLabInformation.__init__(self)
        ResultDescription.__init__(self)
        Result.__init__(self)

    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, exception_traceback):
        pass  # TODO: add rule test

    @property
    def resultMeasureValue(self) -> ResultMeasureValue:
        """
        The reportable measure of the result for the chemical, microbiological or other
        characteristic being analyzed.
        """
        val = Measure(self.resultMeasure)
        return val.resultMeasureValue

    @resultMeasureValue.setter
    def resultMeasureValue(self, val: str) -> None:
        """
        The reportable measure of the result for the chemical, microbiological or other
        characteristic being analyzed.
        """
        tmp = Measure(self.resultMeasure)
        if tmp is None:
            self.resultMeasure = Measure(resultMeasureValue=val)
        else:
            if val is None:
                tmp.resultMeasureValue = None
            else:
                tmp.resultMeasureValue = val
            self.resultMeasure = tmp

    @property
    def resultMeasureUnitCode(self) -> MeasureUnitCode:
        """
        The reportable measure of the result for the chemical, microbiological or other
        characteristic being analyzed.
        """
        val = Measure(self.resultMeasure)
        return val.measureUnitCode

    @resultMeasureUnitCode.setter
    def resultMeasureUnitCode(self, val: MeasureUnitCode) -> None:
        """
        The reportable measure of the result for the chemical, microbiological or other
        characteristic being analyzed.
        """
        tmp = Measure(self.resultMeasure)
        if tmp is None:
            self.resultMeasure = Measure(measureUnitCode=val)
        else:
            if val is None:
                tmp.measureUnitCode = None
            else:
                tmp.measureUnitCode = val
            self.resultMeasure = tmp

    @property
    def resultMeasureQualifierCode(self) -> MeasureQualifierCode:
        """
        The reportable measure of the result for the chemical, microbiological or other
        characteristic being analyzed.
        """
        val = Measure(self.resultMeasure)
        return val.measureQualifierCode

    @resultMeasureQualifierCode.setter
    def resultMeasureQualifierCode(self, val: List[MeasureQualifierCode]) -> None:
        """
        The reportable measure of the result for the chemical, microbiological or other
        characteristic being analyzed.
        """
        tmp = Measure(self.resultMeasure)
        if tmp is None:
            self.resultMeasure = Measure(measureQualifierCode=val)
        else:
            if val is None:
                tmp.measureQualifierCode = None
            else:
                tmp.measureQualifierCode = val
            self.resultMeasure = tmp

    @property
    def resultDepthHeightMeasureValue(self) -> MeasureValue:
        """
        The reportable measure of the result for the chemical, microbiological or other
        characteristic being analyzed.
        """
        val = MeasureCompact(self.resultDepthHeightMeasure)
        return val.measureValue

    @resultDepthHeightMeasureValue.setter
    def resultDepthHeightMeasureValue(self, val: str) -> None:
        """
        The reportable measure of the result for the chemical, microbiological or other
        characteristic being analyzed.
        """
        tmp = MeasureCompact(self.resultDepthHeightMeasure)
        if val is None:
            tmp.measureValue = None
        else:
            tmp.measureValue = val
        self.resultDepthHeightMeasure = tmp

    @property
    def resultDepthHeightMeasureUnitCode(self) -> MeasureUnitCode:
        """
        The reportable measure of the result for the chemical, microbiological or other
        characteristic being analyzed.
        """
        val = MeasureCompact(self.resultDepthHeightMeasure)
        return val.measureUnitCode

    @resultDepthHeightMeasureUnitCode.setter
    def resultDepthHeightMeasureUnitCode(self, val: MeasureUnitCode) -> None:
        """
        The reportable measure of the result for the chemical, microbiological or other
        characteristic being analyzed.
        """
        tmp = MeasureCompact(self.resultDepthHeightMeasure)
        if val is None:
            tmp.measureUnitCode = None
        else:
            tmp.measureUnitCode = val
        self.resultDepthHeightMeasure = tmp



from yattag import Doc

from ..exceptions import WQXException
from .SimpleContent import GearProcedureUnitCode, MeasureValue


class CollectionEffort:
    """
    The fields to describe the effort used a collection.
    """

    __measureValue: MeasureValue
    __gearProcedureUnitCode: GearProcedureUnitCode

    def __init__(
        self,
        o: dict = None,
        *,
        measureValue: MeasureValue = None,
        gearProcedureUnitCode: GearProcedureUnitCode = None
    ):
        if isinstance(o, CollectionEffort):
            # Assign attributes from object without typechecking
            self.__measureValue = o.measureValue
            self.__gearProcedureUnitCode = o.gearProcedureUnitCode
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.measureValue = o.get("measureValue")
            self.gearProcedureUnitCode = o.get("gearProcedureUnitCode")
        else:
            # Assign attributes from named keywords with typechecking
            self.measureValue = measureValue
            self.gearProcedureUnitCode = gearProcedureUnitCode

    @property
    def measureValue(self) -> MeasureValue:
        return self.__measureValue

    @measureValue.setter
    def measureValue(self, val: MeasureValue) -> None:
        self.__measureValue = MeasureValue(val)

    @property
    def gearProcedureUnitCode(self) -> GearProcedureUnitCode:
        return self.__gearProcedureUnitCode

    @gearProcedureUnitCode.setter
    def gearProcedureUnitCode(self, val: GearProcedureUnitCode) -> None:
        self.__gearProcedureUnitCode = GearProcedureUnitCode(val)

    def generateXML(self, name: str = "CollectionEffort") -> str:
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__measureValue is None:
                raise WQXException("Attribute 'measureValue' is required.")
            line("MeasureValue", self.__measureValue)
            if self.__gearProcedureUnitCode is None:
                raise WQXException("Attribute 'gearProcedureUnitCode' is required.")
            line("GearProcedureUnitCode", self.__gearProcedureUnitCode)

        return doc.getvalue()



from yattag import Doc

from ..exceptions import WQXException
from .SimpleContent import (
    ActivityGroupIdentifier,
    ActivityGroupName,
    ActivityGroupTypeCode,
    ActivityIdentifier,
)


class ActivityGroup:
    """
    Allows for the grouping of activities.
    """

    __activityGroupIdentifier: ActivityGroupIdentifier
    __activityGroupName: ActivityGroupName
    __activityGroupTypeCode: ActivityGroupTypeCode
    __activityIdentifier: ActivityIdentifier
    __replaceActivities: bool

    def __init__(
        self,
        o: dict = None,
        *,
        activityGroupIdentifier: ActivityGroupIdentifier = None,
        activityGroupName: ActivityGroupName = None,
        activityGroupTypeCode: ActivityGroupTypeCode = None,
        activityIdentifier: ActivityIdentifier = None,
        replaceActivities: bool = False
    ):
        if isinstance(o, ActivityGroup):
            # Assign attributes from object without typechecking
            self.__activityGroupIdentifier = o.activityGroupIdentifier
            self.__activityGroupName = o.activityGroupName
            self.__activityGroupTypeCode = o.activityGroupTypeCode
            self.__activityIdentifier = o.activityIdentifier
            self.__replaceActivities = o.replaceActivities
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.activityGroupIdentifier = o.get("activityGroupIdentifier")
            self.activityGroupName = o.get("activityGroupName")
            self.activityGroupTypeCode = o.get("activityGroupTypeCode")
            self.activityIdentifier = o.get("activityIdentifier")
            self.replaceActivities = o.get("replaceActivities", False)
        else:
            # Assign attributes from named keywords with typechecking
            self.activityGroupIdentifier = activityGroupIdentifier
            self.activityGroupName = activityGroupName
            self.activityGroupTypeCode = activityGroupTypeCode
            self.activityIdentifier = activityIdentifier
            self.replaceActivities = replaceActivities

    @property
    def activityGroupIdentifier(self) -> ActivityGroupIdentifier:
        return self.__activityGroupIdentifier

    @activityGroupIdentifier.setter
    def activityGroupIdentifier(self, val: ActivityGroupIdentifier) -> None:
        self.__activityGroupIdentifier = (
            None if val is None else ActivityGroupIdentifier(val)
        )

    @property
    def activityGroupName(self) -> ActivityGroupName:
        return self.__activityGroupName

    @activityGroupName.setter
    def activityGroupName(self, val: ActivityGroupName) -> None:
        self.__activityGroupName = None if val is None else ActivityGroupName(val)

    @property
    def activityGroupTypeCode(self) -> ActivityGroupTypeCode:
        return self.__activityGroupTypeCode

    @activityGroupTypeCode.setter
    def activityGroupTypeCode(self, val: ActivityGroupTypeCode) -> None:
        self.__activityGroupTypeCode = None if val is None else ActivityGroupTypeCode(val)

    @property
    def activityIdentifier(self) -> ActivityIdentifier:
        return self.__activityIdentifier

    @activityIdentifier.setter
    def activityIdentifier(self, val: ActivityIdentifier) -> None:
        self.__activityIdentifier = None if val is None else ActivityIdentifier(val)

    @property
    def replaceActivities(self) -> bool:
        return self.__replaceActivities

    @replaceActivities.setter
    def replaceActivities(self, val: bool) -> None:
        self.__replaceActivities = bool(val)

    def generateXML(self, name: str = "ActivityGroup") -> str:
        doc = Doc()
        line = doc.line
        tag = doc.tag

        with tag(name, ("ReplaceActivities", str(self.__replaceActivities))):
            if self.__activityGroupIdentifier is None:
                raise WQXException("Attribute 'activityGroupIdentifier' is required.")
            line("ActivityGroupIdentifier", self.__activityGroupIdentifier)
            if self.__activityGroupName is not None:
                line("ActivityGroupName", self.__activityGroupName)
            if self.__activityGroupTypeCode is None:
                raise WQXException("Attribute 'activityGroupTypeCode' is required.")
            line("ActivityGroupTypeCode", self.__activityGroupTypeCode)
            if len(self.__activityIdentifier) < 2:
                raise WQXException(
                    "Attribute 'activityIdentifier' must be a list of 2 or more "
                    "ActivityIdentifier objects."
                )
            for x in self.__activityIdentifier:
                line("ActivityIdentifier", x)

        return doc.getvalue()



from yattag import Doc

from .CollectionEffort import CollectionEffort
from .MeasureCompact import MeasureCompact
from .NetInformation import NetInformation
from .SimpleContent import CollectionDescriptionText, PassCount


class BiologicalHabitatCollectionInformation:
    """
    Allows for the reporting of biological habitat sample collection information.
    """

    __collectionDuration: MeasureCompact
    __collectionArea: MeasureCompact
    __collectionEffort: CollectionEffort
    __reachLengthMeasure: MeasureCompact
    __reachWidthMeasure: MeasureCompact
    __collectionDescriptionText: CollectionDescriptionText
    __passCount: PassCount
    __netInformation: NetInformation

    def __init__(
        self,
        o: dict = None,
        *,
        collectionDuration: MeasureCompact = None,
        collectionArea: MeasureCompact = None,
        collectionEffort: CollectionEffort = None,
        reachLengthMeasure: MeasureCompact = None,
        reachWidthMeasure: MeasureCompact = None,
        collectionDescriptionText: CollectionDescriptionText = None,
        passCount: PassCount = None,
        netInformation: NetInformation = None
    ):
        if isinstance(o, BiologicalHabitatCollectionInformation):
            # Assign attributes from objects without typechecking
            self.__collectionDuration = o.collectionDuration
            self.__collectionArea = o.collectionArea
            self.__collectionEffort = o.collectionEffort
            self.__reachLengthMeasure = o.reachLengthMeasure
            self.__reachWidthMeasure = o.reachWidthMeasure
            self.__collectionDescriptionText = o.collectionDescriptionText
            self.__passCount = o.passCount
            self.__netInformation = o.netInformation
        elif isinstance(o, dict):
            # Assign attributes from dictionary with typechecking
            self.collectionDuration = o.get("collectionDuration")
            self.collectionArea = o.get("collectionArea")
            self.collectionEffort = o.get("collectionEffort")
            self.reachLengthMeasure = o.get("reachLengthMeasure")
            self.reachWidthMeasure = o.get("reachWidthMeasure")
            self.collectionDescriptionText = o.get("collectionDescriptionText")
            self.passCount = o.get("passCount")
            self.netInformation = o.get("netInformation")
        else:
            # Assign attributes from named keywords with typechecking
            self.collectionDuration = collectionDuration
            self.collectionArea = collectionArea
            self.collectionEffort = collectionEffort
            self.reachLengthMeasure = reachLengthMeasure
            self.reachWidthMeasure = reachWidthMeasure
            self.collectionDescriptionText = collectionDescriptionText
            self.passCount = passCount
            self.netInformation = netInformation

    @property
    def collectionDuration(self) -> MeasureCompact:
        """
        The length of time a collection procedure or protocol was performed (e.g. total
        energized time for electrofishing, or total time kick net used).
        """
        return self.__collectionDuration

    @collectionDuration.setter
    def collectionDuration(self, val: MeasureCompact) -> None:
        """
        The length of time a collection procedure or protocol was performed (e.g. total
        energized time for electrofishing, or total time kick net used).
        """
        self.__collectionDuration = None if val is None else MeasureCompact(val)

    @property
    def collectionArea(self) -> MeasureCompact:
        """
        The area of a collection procedure or protocol was performed (e.g. total area
        coverage for electrofishing, or total area  kick net used).
        """
        return self.__collectionArea

    @collectionArea.setter
    def collectionArea(self, val: MeasureCompact) -> None:
        """
        The area of a collection procedure or protocol was performed (e.g. total area
        coverage for electrofishing, or total area  kick net used).
        """
        self.__collectionArea = None if val is None else MeasureCompact(val)

    @property
    def collectionEffort(self) -> CollectionEffort:
        return self.__collectionEffort

    @collectionEffort.setter
    def collectionEffort(self, val: CollectionEffort) -> None:
        self.__collectionEffort = None if val is None else CollectionEffort(val)

    @property
    def reachLengthMeasure(self) -> MeasureCompact:
        """
        A measurement of the water body length distance in which the procedure or
        protocol was performed.
        """
        return self.__reachLengthMeasure

    @reachLengthMeasure.setter
    def reachLengthMeasure(self, val: MeasureCompact) -> None:
        """
        A measurement of the water body length distance in which the procedure or
        protocol was performed.
        """
        self.__reachLengthMeasure = None if val is None else MeasureCompact(val)

    @property
    def reachWidthMeasure(self) -> MeasureCompact:
        """
        A measurement of the reach width during collection procedures.
        """
        return self.__reachWidthMeasure

    @reachWidthMeasure.setter
    def reachWidthMeasure(self, val: MeasureCompact) -> None:
        """
        A measurement of the reach width during collection procedures.
        """
        self.__reachWidthMeasure = None if val is None else MeasureCompact(val)

    @property
    def collectionDescriptionText(self) -> CollectionDescriptionText:
        return self.__collectionDescriptionText

    @collectionDescriptionText.setter
    def collectionDescriptionText(self, val: CollectionDescriptionText) -> None:
        self.__collectionDescriptionText = (
            None if val is None else CollectionDescriptionText(val)
        )

    @property
    def passCount(self) -> PassCount:
        return self.__passCount

    @passCount.setter
    def passCount(self, val: PassCount) -> None:
        self.__passCount = None if val is None else PassCount(val)

    @property
    def netInformation(self) -> NetInformation:
        return self.__netInformation

    @netInformation.setter
    def netInformation(self, val: NetInformation) -> None:
        self.__netInformation = None if val is None else NetInformation(val)

    def generateXML(self, name: str = "BiologicalHabitatCollectionInformation") -> str:
        doc = Doc()
        asis = doc.asis
        line = doc.line
        tag = doc.tag

        with tag(name):
            if self.__collectionDuration is not None:
                asis(self.__collectionDuration.generateXML("CollectionDuration"))
            if self.__collectionArea is not None:
                asis(self.__collectionArea.generateXML("CollectionArea"))
            if self.__collectionEffort is not None:
                asis(self.__collectionEffort.generateXML("CollectionEffort"))
            if self.__reachLengthMeasure is not None:
                asis(self.__reachLengthMeasure.generateXML("ReachLengthMeasure"))
            if self.__reachWidthMeasure is not None:
                asis(self.__reachWidthMeasure.generateXML("ReachWidthMeasure"))
            if self.__collectionDescriptionText is not None:
                line("CollectionDescriptionText", self.__collectionDescriptionText)
            if self.__passCount is not None:
                line("PassCount", self.__passCount)
            if self.__netInformation is not None:
                asis(self.__netInformation.generateXML("NetInformation"))

        return doc.getvalue()
